/*
 * Copyright 2001-2013 Artima, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.scalatest

import SharedHelpers.thisLineNumber

class StringShouldBeEmptyLogicalOrSpec extends Spec with Matchers {
  
  //ADDITIONAL//
  
  val fileName: String = "StringShouldBeEmptyLogicalOrSpec.scala"
  
  def wasEqualTo(left: Any, right: Any): String = 
    FailureMessages("wasEqualTo", left, right)
    
  def wasNotEqualTo(left: Any, right: Any): String = {
    val (leftee, rightee) = Suite.getObjectsForFailureMessage(left, right)
    FailureMessages("wasNotEqualTo", leftee, rightee)
  }
    
  def equaled(left: Any, right: Any): String = 
    FailureMessages("equaled", left, right)
    
  def didNotEqual(left: Any, right: Any): String = {
    val (leftee, rightee) = Suite.getObjectsForFailureMessage(left, right)
    FailureMessages("didNotEqual", leftee, rightee)
  }
  
  def wasNotEmpty(left: Any): String = 
    FailureMessages("wasNotEmpty", left)
    
  def wasEmpty(left: Any): String = 
    FailureMessages("wasEmpty", left)
    
  def allError(message: String, lineNumber: Int, left: Any): String = {
    val messageWithIndex = UnquotedString("  " + FailureMessages("forAssertionsGenTraversableMessageWithStackDepth", 0, UnquotedString(message), UnquotedString(fileName + ":" + lineNumber)))
    FailureMessages("allShorthandFailed", messageWithIndex, left)
  }
    
  val nonEmptyString = "123"
  val emptyString = ""
  
  object `Sorted matcher` {
    
    object `when work with 'list should be (empty)'` {
      
      def `should do nothing when list is empty` {
        
        emptyString should (be (empty) or be (emptyString))
        nonEmptyString should (be (empty) or be (nonEmptyString))
        emptyString should (be (empty) or be (nonEmptyString))
        
        emptyString should (be (emptyString) or be (empty))
        emptyString should (be (nonEmptyString) or be (empty))
        nonEmptyString should (be (nonEmptyString) or be (empty))
        
        emptyString should (be (empty) or equal (emptyString))
        nonEmptyString should (be (empty) or equal (nonEmptyString))
        emptyString should (be (empty) or equal (nonEmptyString))
        
        emptyString should (equal (emptyString) or be (empty))
        emptyString should (equal (nonEmptyString) or be (empty))
        nonEmptyString should (equal (nonEmptyString) or be (empty))
      }
      
      def `should throw TestFailedException with correct stack depth when file is not empty` {
        val caught1 = intercept[TestFailedException] {
          nonEmptyString should (be (empty) or be (emptyString))
        }
        assert(caught1.message === Some(wasNotEmpty(nonEmptyString) + ", and " + wasNotEqualTo(nonEmptyString, emptyString)))
        assert(caught1.failedCodeFileName === Some(fileName))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught2 = intercept[TestFailedException] {
          nonEmptyString should (be (emptyString) or be (empty))
        }
        assert(caught2.message === Some(wasNotEqualTo(nonEmptyString, emptyString) + ", and " + wasNotEmpty(nonEmptyString)))
        assert(caught2.failedCodeFileName === Some(fileName))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught3 = intercept[TestFailedException] {
          nonEmptyString should (be (empty) or equal (emptyString))
        }
        assert(caught3.message === Some(wasNotEmpty(nonEmptyString) + ", and " + didNotEqual(nonEmptyString, emptyString)))
        assert(caught3.failedCodeFileName === Some(fileName))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught4 = intercept[TestFailedException] {
          nonEmptyString should (equal (emptyString) or be (empty))
        }
        assert(caught4.message === Some(didNotEqual(nonEmptyString, emptyString) + ", and " + wasNotEmpty(nonEmptyString)))
        assert(caught4.failedCodeFileName === Some(fileName))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
    }
    
    object `when work with 'file should not be empty'` {
      
      def `should do nothing when file is not empty` {
        nonEmptyString should (not be empty or not be emptyString)
        emptyString should (not be empty or not be nonEmptyString)
        nonEmptyString should (not be empty or not be nonEmptyString)
        
        nonEmptyString should (not be emptyString or not be empty)
        nonEmptyString should (not be nonEmptyString or not be empty)
        emptyString should (not be nonEmptyString or not be empty)
        
        nonEmptyString should (not be empty or not equal emptyString)
        emptyString should (not be empty or not equal nonEmptyString)
        nonEmptyString should (not be empty or not equal nonEmptyString)
        
        nonEmptyString should (not equal emptyString or not be empty)
        nonEmptyString should (not equal nonEmptyString or not be empty)
        emptyString should (not equal nonEmptyString or not be empty)
      }
      
      def `should throw TestFailedException with correct stack depth when file is empty` {
        val caught1 = intercept[TestFailedException] {
          emptyString should (not be empty or not be emptyString)
        }
        assert(caught1.message === Some(wasEmpty(emptyString) + ", and " + wasEqualTo(emptyString, emptyString)))
        assert(caught1.failedCodeFileName === Some(fileName))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught2 = intercept[TestFailedException] {
          emptyString should (not be emptyString or not be empty)
        }
        assert(caught2.message === Some(wasEqualTo(emptyString, emptyString) + ", and " + wasEmpty(emptyString)))
        assert(caught2.failedCodeFileName === Some(fileName))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught3 = intercept[TestFailedException] {
          emptyString should (not be empty or not equal emptyString)
        }
        assert(caught3.message === Some(wasEmpty(emptyString) + ", and " + equaled(emptyString, emptyString)))
        assert(caught3.failedCodeFileName === Some(fileName))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught4 = intercept[TestFailedException] {
          emptyString should (not equal emptyString or not be empty)
        }
        assert(caught4.message === Some(equaled(emptyString, emptyString) + ", and " + wasEmpty(emptyString)))
        assert(caught4.failedCodeFileName === Some(fileName))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
    }
    
    object `when work with 'all(xs) should be (empty)'` {
      
      def `should do nothing when all(xs) is empty` {
        all(Vector(emptyString)) should (be (empty) or be (emptyString))
        all(Vector(nonEmptyString)) should (be (empty) or be (nonEmptyString))
        all(Vector(emptyString)) should (be (empty) or be (nonEmptyString))
        
        all(Vector(emptyString)) should (be (emptyString) or be (empty))
        all(Vector(emptyString)) should (be (nonEmptyString) or be (empty))
        all(Vector(nonEmptyString)) should (be (nonEmptyString) or be (empty))
        
        all(Vector(emptyString)) should (be (empty) or equal (emptyString))
        all(Vector(nonEmptyString)) should (be (empty) or equal (nonEmptyString))
        all(Vector(emptyString)) should (be (empty) or equal (nonEmptyString))
        
        all(Vector(emptyString)) should (equal (emptyString) or be (empty))
        all(Vector(emptyString)) should (equal (nonEmptyString) or be (empty))
        all(Vector(nonEmptyString)) should (equal (nonEmptyString) or be (empty))
      }
      
      def `should throw TestFailedException with correct stack depth when xs is not sorted` {
        val left1 = Vector(nonEmptyString)
        val caught1 = intercept[TestFailedException] {
          all(left1) should (be (emptyString) or be (empty))
        }
        assert(caught1.message === Some(allError(wasNotEqualTo(nonEmptyString, emptyString) + ", and " + wasNotEmpty(nonEmptyString), thisLineNumber - 2, left1)))
        assert(caught1.failedCodeFileName === Some(fileName))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left2 = Vector(nonEmptyString)
        val caught2 = intercept[TestFailedException] {
          all(left2) should (be (empty) or be (emptyString))
        }
        assert(caught2.message === Some(allError(wasNotEmpty(nonEmptyString) + ", and " + wasNotEqualTo(nonEmptyString, emptyString), thisLineNumber - 2, left2)))
        assert(caught2.failedCodeFileName === Some(fileName))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left3 = Vector(nonEmptyString)
        val caught3 = intercept[TestFailedException] {
          all(left3) should (equal (emptyString) or be (empty))
        }
        assert(caught3.message === Some(allError(didNotEqual(nonEmptyString, emptyString) + ", and " + wasNotEmpty(nonEmptyString), thisLineNumber - 2, left3)))
        assert(caught3.failedCodeFileName === Some(fileName))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left4 = Vector(nonEmptyString)
        val caught4 = intercept[TestFailedException] {
          all(left4) should (be (empty) or equal (emptyString))
        }
        assert(caught4.message === Some(allError(wasNotEmpty(nonEmptyString) + ", and " + didNotEqual(nonEmptyString, emptyString), thisLineNumber - 2, left4)))
        assert(caught4.failedCodeFileName === Some(fileName))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
    }
    
    object `when work with 'all(xs) should not be sorted'` {
      def `should do nothing when xs is not sorted` {
        all(Vector(nonEmptyString)) should (not be empty or not be emptyString)
        all(Vector(emptyString)) should (not be empty or not be nonEmptyString)
        all(Vector(nonEmptyString)) should (not be empty or not be nonEmptyString)
        
        all(Vector(nonEmptyString)) should (not be emptyString or not be empty)
        all(Vector(nonEmptyString)) should (not be nonEmptyString or not be empty)
        all(Vector(emptyString)) should (not be nonEmptyString or not be empty)
        
        all(Vector(nonEmptyString)) should (not be empty or not equal emptyString)
        all(Vector(emptyString)) should (not be empty or not equal nonEmptyString)
        all(Vector(nonEmptyString)) should (not be empty or not equal nonEmptyString)
        
        all(Vector(nonEmptyString)) should (not equal emptyString or not be empty)
        all(Vector(nonEmptyString)) should (not equal nonEmptyString or not be empty)
        all(Vector(emptyString)) should (not equal nonEmptyString or not be empty)
      }
      
      def `should throw TestFailedException with correct stack depth when xs is not sorted` {
        val left1 = Vector(emptyString)
        val caught1 = intercept[TestFailedException] {
          all(left1) should (not be emptyString or not be empty)
        }
        assert(caught1.message === Some(allError(wasEqualTo(emptyString, emptyString) + ", and " + wasEmpty(emptyString), thisLineNumber - 2, left1)))
        assert(caught1.failedCodeFileName === Some(fileName))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left2 = Vector(emptyString)
        val caught2 = intercept[TestFailedException] {
          all(left2) should (not be empty or not be emptyString)
        }
        assert(caught2.message === Some(allError(wasEmpty(emptyString) + ", and " + wasEqualTo(emptyString, emptyString), thisLineNumber - 2, left2)))
        assert(caught2.failedCodeFileName === Some(fileName))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left3 = Vector(emptyString)
        val caught3 = intercept[TestFailedException] {
          all(left3) should (not equal emptyString or not be empty)
        }
        assert(caught3.message === Some(allError(equaled(emptyString, emptyString) + ", and " + wasEmpty(emptyString), thisLineNumber - 2, left3)))
        assert(caught3.failedCodeFileName === Some(fileName))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left4 = Vector(emptyString)
        val caught4 = intercept[TestFailedException] {
          all(left4) should (not be empty or not equal emptyString)
        }
        assert(caught4.message === Some(allError(wasEmpty(emptyString) + ", and " + equaled(emptyString, emptyString), thisLineNumber - 2, left4)))
        assert(caught4.failedCodeFileName === Some(fileName))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
    }
  }
}
